package com.dsk.flink.dsc.common.sink;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.db.DbUtil;
import com.alibaba.druid.pool.DruidDataSource;
import com.dsk.flink.dsc.common.dto.SqlErrorLog;
import com.dsk.flink.dsc.utils.EnvProperties;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.Date;
import java.util.concurrent.*;

public class MysqlDataTransferSink extends RichSinkFunction<String> {

    static Logger logger = LoggerFactory.getLogger(MysqlDataTransferSink.class);
    EnvProperties envProps;
    ExecutorService executorService;
    DruidDataSource dataSource;

    public MysqlDataTransferSink(EnvProperties envProps) {
        this.envProps = envProps;
    }

    @Override
    public void open(Configuration parameters) throws Exception {

        executorService = new ThreadPoolExecutor(4, 4, 20, TimeUnit.MINUTES, new LinkedBlockingDeque<>());
        //初始化获取配置
        String configTidbUrl = String.format(envProps.getDb_url(), envProps.getDb_host(), envProps.getDb_port(), envProps.getDb_database());
        //System.out.println(configTidbUrl);
        dataSource = new DruidDataSource();
        dataSource.setDriverClassName("com.mysql.cj.jdbc.Driver");
        dataSource.setUsername(envProps.getDb_username());
        dataSource.setPassword(envProps.getDb_password());
        dataSource.setUrl(configTidbUrl);
        dataSource.setMaxActive(30);
        dataSource.setInitialSize(10);
        dataSource.setTestWhileIdle(true);
        dataSource.setMaxWait(20000);
        dataSource.setValidationQuery("select 1");
    }

    @Override
    public void close() throws Exception {
        executorService.shutdown();
        dataSource.close();
    }

    @Override
    public void invoke(String value, Context context) throws Exception {
        executorService.execute(() -> {
            try {
                executeSql(value);
            }catch (Exception e){
                e.printStackTrace();
            }
        });
    }

    private void executeSql(String sql) throws Exception{
        Connection connection = null;
        PreparedStatement pt = null;
        try {
            connection = dataSource.getConnection();
            pt = connection.prepareStatement(sql);
            pt.execute();
        } catch (Exception e) {
            System.out.println("sql报错----->" + sql);
            e.printStackTrace();
            logger.error("------错误时间:{}-----,sql:{}--------异常:{}", DateUtil.now(),sql,e.getMessage());
            SqlErrorLog errorLog = new SqlErrorLog(new Date(), sql, e.getMessage());
            try {
                writeErrLog(errorLog);
            }catch (Exception re){
                logger.error("错误日志保存异常 -> {}", re.getMessage());
            }
        } finally {
            if (pt != null) {
                pt.close();
            }
            if (connection != null) {
                connection.close();
            }
        }
    }

    private void writeErrLog(SqlErrorLog errorLog) {
        writeErrLogDb(errorLog);
    }

    private void writeErrLogDb(SqlErrorLog errorLog) {
        Snowflake snowflake = IdUtil.getSnowflake(RandomUtil.randomInt(31), RandomUtil.randomInt(31));
        String sql = "insert dsc_err_log (id,error_time, error_sql, error_msg) values (?, ?, ?, ?)";
        Connection conn = null;
        PreparedStatement pt = null;
        try {
            conn = dataSource.getConnection();
            pt = conn.prepareStatement(sql);
            pt.setLong(1, snowflake.nextId() + RandomUtil.randomInt(10,99));
            pt.setObject(2, errorLog.getErrorTime());
            pt.setString(3, errorLog.getSql());
            pt.setString(4, errorLog.getError());
            pt.execute();
        }catch (Exception e){
            logger.error("错误日志保存异常 -> {}", e.getMessage());
        }finally {
            DbUtil.close(conn, pt);
        }
    }
}
