package com.dsk.flink.dsc.utils;

import cn.hutool.core.util.StrUtil;

import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.util.AbstractMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.jar.JarFile;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;

/**
 * @Description: 
 * @author zhaowei 2022年9月29日
 */
public class EnvPropertiesUtil {

	private static String SYSTEM_ENV = "pro";
	private static EnvProperties properties = new EnvProperties();
	static {
		SYSTEM_ENV = System.getProperty("profiles.active") == null ? SYSTEM_ENV : System.getProperty("profiles.active");
		System.out.println("Env: " + SYSTEM_ENV + " properties");
		URI uri = null;
		try {
			uri = EnvPropertiesUtil.class.getProtectionDomain().getCodeSource().getLocation().toURI();
		} catch (URISyntaxException e1) {
			e1.printStackTrace();
		}
		
		if (uri.toString().endsWith(".jar")) {
			// jar包内
			String jarPath = uri.toString();
			uri = URI.create(jarPath.substring(jarPath.indexOf("file:"),jarPath.indexOf(".jar") + 4));
			try {
				allPropertiesJar(uri, properties);
			}catch (Exception e) {
				e.printStackTrace();
			}
		} else {
			//本地
			try {
				allPropertiesLocal(uri, properties);
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
	}



	/**
	 * 
	 * @Description: 获取默认配置
	 * @param key
	 * @return
	 */
	public static EnvProperties getProperties() {
		return (EnvProperties) properties.clone();
	}
	
	/**
	 * 
	 * @Description: 获取配置值
	 * @param key
	 * @return
	 */
	public static String getProperty(String key) {
		return properties.getProperty(key);
	}

	/**
	 * 本地环境读资源
	 * @throws IOException 
	 */
	private static void allPropertiesLocal(URI uri, Properties properties) throws IOException {
		File resources = new File(uri.getPath());
		if (!resources.exists() || !resources.isDirectory()) {
            return ;
        }
		File[] propertiesFiles = resources.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				if(!name.endsWith(".properties")) return false;
				if(name.matches(getApplicationPropertiesRegx()) && !name.equals(getApplicationPropertiesName())) {
					return false;
				}
				return true;
			}
		});
		if(propertiesFiles == null || propertiesFiles.length == 0) {
			return ;
		}
		for(File file : propertiesFiles) {
			properties.load(new FileInputStream(file));
		}
	}
	/**
	 * Jar环境读资源
	 * @throws IOException 
	 */
	private static void allPropertiesJar(URI uri, Properties properties) throws IOException {
		List<Map.Entry<ZipEntry, InputStream>> collect = readJarFile(new JarFile(uri.getPath())).collect(Collectors.toList());
		if(collect == null || collect.isEmpty()) {
			return;
		}
        for (Map.Entry<ZipEntry, InputStream> entry : collect) {
            // 文件相对路径
            String key = entry.getKey().getName();
            // 文件流
            InputStream stream = entry.getValue();
            properties.load(stream);
        }
	}
	private static Stream<Map.Entry<ZipEntry, InputStream>> readJarFile(JarFile jarFile) {
		Stream<Map.Entry<ZipEntry, InputStream>> readingStream = jarFile.stream()
				.filter(entry -> {
					if(entry.getName().matches(getApplicationPropertiesRegx()) && !entry.getName().equals(getApplicationPropertiesName())) {
						return false;
					}
					return !entry.isDirectory() && entry.getName().endsWith(".properties");
				})
				.map(entry -> {
					try {
						return new AbstractMap.SimpleEntry<>(entry, jarFile.getInputStream(entry));
					} catch (IOException e) {
						return new AbstractMap.SimpleEntry<>(entry, null);
					}
				});
		return readingStream.onClose(() -> {
			try {
				jarFile.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		});
	}

	public static EnvProperties getPropertiesFromArgsPath(String filePath) throws IOException {
		EnvProperties envProperties = new EnvProperties();
		if(StrUtil.isBlank(filePath)){

			filePath = System.getProperties().getProperty("os.name").contains("Windows") ? "D:\\Env\\application.properties" : "/home/module/flink-job/application.properties";
		}
		File file = new File(filePath);
		if (!file.exists()) {
			return new EnvProperties();
		}
		envProperties.load(Files.newInputStream(file.toPath()));
		//System.out.println("获取到的kafka_topic为："+envProperties.getKafka_topic());
		return envProperties;
	}

	private static String getApplicationPropertiesName() {
		return "application_"+SYSTEM_ENV+".properties";
	}
	private static String getApplicationPropertiesRegx() {
		return "application_[a-z]{3,5}\\.properties";
	}

}
